/**************************************************************************
*  Copyright (c) 2014-2015 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  14.06.2014  mifi  First Version.
*  15.05.2015  mifi  Move GPMC functionality to b3gpmc. 
**************************************************************************/
#define __HW_C__

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include <stdint.h>
#include <ctl_api.h>
#include "hw.h"
#include "b3gpmc.h"

#include "soc_AM335x.h"
#include "hw_control_AM335x.h"
#include "hw_types.h"
#include "hw_cm_per.h"
#include "hw_gpio_v2.h"
#include "gpmc.h"

/*=======================================================================*/
/*  All extern data                                                      */
/*=======================================================================*/

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Control register
 */
#define CONTROL_CONF_MMODE_SHIFT       0
#define CONTROL_CONF_PUDEN_SHIFT       3
#define CONTROL_CONF_PUTYPESEL_SHIFT   4
#define CONTROL_CONF_RXACTIVE_SHIFT    5

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/   

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static volatile uint32_t SysTick = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  GPIO1ModuleClkConfig                                                 */
/*                                                                       */
/*  This function enables the functional clocks for GPIO1 instance.      */
/*  Based on the StarterWare example (platform\evmskAM335x\gpio.c).      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void GPIO1ModuleClkConfig (void)
{
   /* Writing to MODULEMODE field of CM_PER_GPIO1_CLKCTRL register. */
   HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) |=
         CM_PER_GPIO1_CLKCTRL_MODULEMODE_ENABLE;

   /* Waiting for MODULEMODE field to reflect the written value. */
   while(CM_PER_GPIO1_CLKCTRL_MODULEMODE_ENABLE !=
         (HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) &
          CM_PER_GPIO1_CLKCTRL_MODULEMODE));
          
   /*
   ** Writing to OPTFCLKEN_GPIO_1_GDBCLK bit in CM_PER_GPIO1_CLKCTRL
   ** register.
   */
   HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) |=
         CM_PER_GPIO1_CLKCTRL_OPTFCLKEN_GPIO_1_GDBCLK;

   /*
   ** Waiting for OPTFCLKEN_GPIO_1_GDBCLK bit to reflect the desired
   ** value.
   */
   while(CM_PER_GPIO1_CLKCTRL_OPTFCLKEN_GPIO_1_GDBCLK !=
         (HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) &
          CM_PER_GPIO1_CLKCTRL_OPTFCLKEN_GPIO_1_GDBCLK));

   /*
   ** Waiting for IDLEST field in CM_PER_GPIO1_CLKCTRL register to attain the
   ** desired value.
   */
   while((CM_PER_GPIO1_CLKCTRL_IDLEST_FUNC <<
          CM_PER_GPIO1_CLKCTRL_IDLEST_SHIFT) !=
          (HWREG(SOC_CM_PER_REGS + CM_PER_GPIO1_CLKCTRL) &
           CM_PER_GPIO1_CLKCTRL_IDLEST));

   /*
   ** Waiting for CLKACTIVITY_GPIO_1_GDBCLK bit in CM_PER_L4LS_CLKSTCTRL
   ** register to attain desired value.
   */
   while(CM_PER_L4LS_CLKSTCTRL_CLKACTIVITY_GPIO_1_GDBCLK !=
         (HWREG(SOC_CM_PER_REGS + CM_PER_L4LS_CLKSTCTRL) &
          CM_PER_L4LS_CLKSTCTRL_CLKACTIVITY_GPIO_1_GDBCLK));
          
} /* GPIO1ModuleClkConfig */

/*************************************************************************/
/*  SysTickTimer                                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SysTickTimer (void)
{
   SysTick++;
} /* onTimer */

/***************************************************************************/
/*  LEDConfig                                                              */
/*                                                                         */
/*  Configure he LEDs of the BeagleBone.                                   */
/*  The user LEDs are connected to GPIO1_21 to GPIO1_24.                   */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDConfig (void)
{
   uint32_t Value;
   
   /* Setup Pad Control Registers */
   Value = (7 << CONTROL_CONF_MMODE_SHIFT)     |   /* Mode 7 */
           (1 << CONTROL_CONF_PUDEN_SHIFT)     |   /* Pull disabled */
           (1 << CONTROL_CONF_PUTYPESEL_SHIFT) |   /* Pullup */
           (0 << CONTROL_CONF_RXACTIVE_SHIFT);     /* Receiver disabled */
   
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_A(5)) = Value;
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_A(6)) = Value;
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_A(7)) = Value;
   HWREG(SOC_CONTROL_REGS + CONTROL_CONF_GPMC_A(8)) = Value;
   
   /* Module is enabled, clocks are not gated. */
   HWREG(SOC_GPIO_1_REGS + GPIO_CTRL) &= ~GPIO_CTRL_DISABLEMODULE;

   /* Clear the corresponding bits in the GPIO_DATAOUT register, set outputs to low. */
   HWREG(SOC_GPIO_1_REGS + GPIO_CLEARDATAOUT) = ((1<<21) | (1<<22) | (1<<23) | (1<<24));
   
   /* The corresponding GPIO ports are configured as an output. */
   HWREG(SOC_GPIO_1_REGS + GPIO_OE) &= ~((1<<21) | (1<<22) | (1<<23) | (1<<24));

} /* LEDConfig */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  HWInit                                                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void HWInit (void)
{
   /* Init the general purpose memory controller */
   GPMCInit();
   
   /* Init GPIO clocks */
   GPIO1ModuleClkConfig();    
          
   /* Configure the LEDs of the BeagleBone */
   LEDConfig();
   
   /* Configure SysTick timer */
   ctl_start_timer(SysTickTimer);
   ctl_global_interrupts_enable();   
   
   /* Assert / Deassert RESET on GPMC bus */
   GPMCResetSet();
   HWDelayMs(10);
   GPMCResetClear();
   
} /* HWInit */

/*************************************************************************/
/*  HWDelayMs                                                            */
/*                                                                       */
/*  Very simple delay in milliseconds.                                   */
/*                                                                       */
/*  In    : Delay                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void HWDelayMs (uint32_t Delay)
{
   uint32_t Time;
   
   Time = SysTick + Delay;
   while (SysTick < Time)
   {
      __asm("nop");
   }
   
} /* HWDelayMs */

/*** EOF ***/
