/****************************************************************************
*  Copyright (c) 2014-2015 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************
*  History:
*
*  14.06.2014  mifi  First Version.
*  15.05.2015  mifi  Rework GPMC functionality.
*  07.11.2015  mifi  Corrected project for CrossWorks for ARM v2.3.5.
****************************************************************************/
#define __MAIN_C__

#include <stdint.h>
#include "hw.h"

#include "soc_AM335x.h"
#include "hw_types.h"
#include "hw_gpio_v2.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*
 * Register definition
 */
typedef struct
{
  volatile uint16_t DATAOUT;  /* GPIO data out register */
  volatile uint16_t DATAIN;   /* GPIO data in register */
  volatile uint16_t SET;      /* GPIO data out set register */
  volatile uint16_t CLEAR;    /* GPIO data out clear register */
  
  volatile uint16_t RES4;     /* Reserve */
  volatile uint16_t RES5;     /* Reserve */
  volatile uint16_t RES6;     /* Reserve */
  volatile uint16_t RES7;     /* Reserve */
} GPIO_TypeDef;

#define USER_LED1_ON()  HWREG(SOC_GPIO_1_REGS + GPIO_SETDATAOUT) = (1<<21)
#define USER_LED1_OFF() HWREG(SOC_GPIO_1_REGS + GPIO_CLEARDATAOUT) = (1<<21)

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

static GPIO_TypeDef* GPIO0 = (GPIO_TypeDef*)(GPMC_CS0_START_ADDRESS + 0 * sizeof(GPIO_TypeDef));
static GPIO_TypeDef* GPIO1 = (GPIO_TypeDef*)(GPMC_CS0_START_ADDRESS + 1 * sizeof(GPIO_TypeDef));

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/

/***************************************************************************/
/*  main                                                                   */
/***************************************************************************/
int main (void)
{
   uint16_t Index;
   uint16_t Index1;
   uint16_t Index2;
   
   HWInit();
   
   /* Test data in */
   GPIO0->DATAOUT = 0x07;
   Index1 = GPIO1->DATAIN;
   Index2 = GPIO1->DATAIN;
   GPIO0->DATAOUT = 0x00;
   
   (void)Index1;
   (void)Index2;

   while (1)
   {
      /* Set LED */
      USER_LED1_ON();
      for(Index=0; Index<7; Index++)
      {
         GPIO0->SET = (1<<Index);
         HWDelayMs(100);
      }   
      
      /* Clear LED */
      USER_LED1_OFF();
      for(Index=0; Index<7; Index++)
      {
         GPIO0->CLEAR = (1<<Index);
         HWDelayMs(100);
      }   
   }

   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    *    return type of 'main' is not 'int'
    * we use an int as return :o)
    */ 
   return(0);
} /* main */

/*** EOF ***/
